#include "re.h"
#include <stdio.h>
// #include <stdlib.h>
#include <string.h>

// OUTPUT ****************************************************************
FMT RE_IO::fmt = F_long;

#if   defined(mplib_pari)
    /* On s'inspire de GENtostring, mais en y rajoutant des paramtres */
    #define STEPSIZE 1024
    typedef struct outString {
      char *string;
      ulong len,size;
    } outString;
    static outString OutStr;

    static void
    check_output_length(long len)
    {
      if (OutStr.len + len >= OutStr.size) {
        len += OutStr.size + STEPSIZE;
        OutStr.string = gprealloc(OutStr.string, len, OutStr.size);
        OutStr.size = len;
      }
    }
    #undef STEPSIZE

    static void
    outstr_putc(char c)
    {
      check_output_length(1);
      OutStr.string[OutStr.len++] = c;
    }

    static void
    outstr_puts(char *s)
    {
      long len=strlen(s);
      check_output_length(len);
      strcpy(OutStr.string+OutStr.len,s);
      OutStr.len += len;
    }

    static char *
    SS(FMT p, const RE& a)
    {
      GEN r = a.value();
      /* On s'inspire de GENtostring */
      PariOUT* save = pariOut;
      PariOUT pariOut2Str = { outstr_putc, outstr_puts };
      if (r   == NULL) return strdup("0x0"); 
      pariOut = &pariOut2Str;
      OutStr.len = 0; OutStr.size=0; OutStr.string=NULL;
      switch (p) {
        case F_pari:
        case F_long:   brute(r, 'f', -1); break;
        case F_medium: brute(r, 'g',  8); break;
        case F_short:  brute(r, 'f',  2); break;
      }
      OutStr.string[OutStr.len] = 0;
      pariOut = save;
      return OutStr.string;
    }

#elif defined(mplib_ntl)

    #define L10SL2 (3.321928095)
    static char *
    SS(FMT p, const RE& a)
    {
      char *t, *s;
      RE b, c, d;
      ZZ B, C, Q;
      int QQ;
      long k, l;
      if (IsZero(a)) return strdup("0");
      b = a;
      if (a < RE(0)) negate(b, b);
      l = b.OutputPrecision();
      k = l - long((NumBits(b.mantissa())+b.exponent())/L10SL2);;
      power(c, 10, k);
      mul(b, b, c);
      power(c, 10, l);
      sub(d, c, 1);
      while (b <  c) { k++; mul(b, b, 10); }
      while (b >  d) { k--; div(b, b, 10); }
      add(b, b, 0.5);
      B << b;
      C << c;
      div(C, C, 10);
      long outputlen =
        ( p == F_pari  ) ? 3 + l + (l<k?k-l:0) + (k<0?-k:0) :
        ( p == F_long  ) ? 3 + l + (l<k?k-l:0) + (k<0?-k:0) :
        ( p == F_medium) ? 3 + 5 + 1+long(log(abs(l-k))/log(10)) :
        200 ;
      t = s = (char *) malloc( outputlen );
      if    ( a < RE(0) ) *t++ = '-';
      switch (p) {
      case F_pari:
      case F_long:
        if    ( l < k ) *t++ = '0';
        while ( l < k ) { l++; mul(C, C, 10); }
        while ( l > 0 ) {
          DivRem(Q, B, B, C);
          QQ << Q;
          if (Q > 9) {  *t++ = '0'+(QQ/10); QQ %= 10; } // Hack....
          if (l == k)   *t++ = '.';
          *t++ = '0' + QQ;
          l--; div(C, C, 10);
        }
        while ( k < 0 ) { k++; *t++ = 'o'; }
        break;
      case F_medium:
        l -= k + 1;
        for (k = 0; k < 5 + 2*(!l); k++) { // 5 si exposant, 7 sinon
          if (k == 1)   *t++ = '.';
          DivRem(Q, B, B, C);
          QQ << Q;      *t++ = '0' + QQ;
          div(C, C, 10);
        }
        if (!l) break;
        *t++ = 'e';
        if    ( l < 0 ) *t++ = '-';
        if    ( l < 0 ) l = -l;
        for (k = 1; k < l/10; k *= 10) ;
        while ( k > 0 ) {
          *t++ = '0' + l / k;
          l = l % k; k /= 10;
        }
        break;
      case F_short:
        // A faire...
        break;
      }
      *t++ = 0;
      return s;
    }
    #undef L10SL2

#endif


void operator<<(char*& s, const RE& r) //
{
  static char string[64];
  double d;
  switch (RE_IO::output()) {
    case F_short:
      d << r;
      if (d>1e10) sprintf(string,"%.1g",  d);
      else        sprintf(string,"%4.1f", d);
      s = strdup(string);
      return;
    case F_medium:
      d << r;
      if (d!=0.) {
        sprintf(string,"%g",    d);
        s = strdup(string);
        return;
      }
      // Si l'approximation en "double" est nulle,
      // le nombre lui-mme peut-tre pas.
    #if   defined(mplib_ntl)
    case F_pari:
    case F_long:
      s = SS(RE_IO::output(),r);
      return;
    #elif defined(mplib_pari)
    case F_pari:
    case F_long:
      s = SS(RE_IO::output(),r);
      return;
    #else
    // case F_medium:
      s = strdup("0.");
      return;
    case F_pari:
    case F_long:
      d << r;
      if (d>1e10) sprintf(string,"%.1g",  d);
      else        sprintf(string,"%.12f", d);
      s = strdup(string);
      return;
    #endif
    default:
      s = NULL;
  }
}

// ****************************************************************

#ifndef mplib_pari              /* proto */
#include <sys/time.h>
#ifdef __alpha__
#define __sys_resource_h_recursive
#endif
#include <sys/resource.h>
#ifndef __linux__
extern
  #ifdef __cplusplus
  "C"
  #endif
  int getrusage  __((int who, struct rusage *rusage));
#endif
long
timer2(void)
{
  static long oldmusec, oldsec;
  long delay;
  struct rusage r;
  struct timeval t;
  getrusage(RUSAGE_SELF,&r); t=r.ru_utime;
  delay = 1000 * (t.tv_sec - oldsec) + (t.tv_usec - oldmusec) / 1000;
  oldmusec = t.tv_usec; oldsec = t.tv_sec;
  return delay;
}
#endif                          /* proto */
