#ifndef RE__H
#define RE__H
#include <stdio.h>

class RE
{
  double v;
public:
  // Construction and conversion operations
  ~RE () { }
  RE () { v = NaN; }
  RE (int a) : v(a) { }
  RE (long a) : v(a) { }
  RE (float a) : v(a) { }
  RE (double a) : v(a) { }
  RE (const RE& a) { v = a.v; }
  RE (const char* a);
  void operator = (const RE& a) { this->v = a.v; }
  operator double (void) const;
  operator char* (void) const;
  // Fundamental comparison operators
  friend bool operator == (const RE& a, const RE& b);
  friend bool operator <= (const RE& a, const RE& b);
  friend bool IsZero (const RE& a);
  friend bool IsOne  (const RE& a);
  // Fundamental arithmetic operators
  friend RE operator - (const RE& a);
  friend RE inv   (const RE& a);
  friend RE sqrt  (const RE& a);
  friend RE root  (const RE& a, const unsigned long exposant);
  RE& operator += (const RE& a);
  RE& operator *= (const RE& a);
  RE& operator -= (const RE& a) { *this += -a; return *this; }
  RE& operator /= (const RE& a) { *this *= inv(a); return *this; }
  // Precision
  friend double log2 (const RE& a);    // log en base 2
  friend long precision (const RE& a); // prcision en bit
};

// Construction and conversion operations
inline RE::RE (const char* a) { sscanf(a, "%lf", &v); }
inline RE::operator double (void) const { return v; }
// [re-io.c] inline RE::operator char* (void) const;
// Fundamental comparison operators
inline bool operator == (const RE& a, const RE& b) { return a.v == b.v; }
inline bool operator <= (const RE& a, const RE& b) { return a.v <= b.v; }
inline bool IsZero (const RE& a) { return a.v == 0.; }
inline bool IsOne  (const RE& a) { return a.v == 1.; }
// Fundamental arithmetic operators
inline RE operator - (const RE& a) { RE r; r.v = - a.v; return r; }
inline RE inv  (const RE& a) { RE r; r.v = 1./a.v; return r; }
inline RE sqrt (const RE& a) { RE r; r.v = sqrt(a.v); return r; }
inline RE root (const RE& a, const unsigned long exposant)
  { RE r; r.v = pow(a.v,1./exposant); return r; }
inline RE& RE::operator += (const RE& a) { v += a.v; return *this; }
inline RE& RE::operator *= (const RE& a) { v *= a.v; return *this; }
// Precision
inline double log2 (const RE& a) { return log(fabs(a.v))/log(2.); }
inline long precision (const RE& a) { return 50; }

// -------- Definitions from fundamental methods --------
// Standard arithmetic operators
inline RE operator + (const RE& a, const RE& b) {RE r = a;r += b; return r;}
inline RE operator - (const RE& a, const RE& b) {RE r = a;r -= b; return r;}
inline RE operator * (const RE& a, const RE& b) {RE r = a;r *= b; return r;}
inline RE operator / (const RE& a, const RE& b) {RE r = a;r /= b; return r;}
// Other operators
inline RE sqr (const RE& a) { return a * a; }
inline bool operator != (const RE& a, const RE& b) { return !(a == b); }

#endif
