#ifndef H_POLYNOME
#define H_POLYNOME

#include "cx.h"
#include "vecteur.h"

class POLYNOME
{
  //[GNUC] static const ulong chunk = 10;
  enum { chunk = 10 };
  ulong len, deg;
  CX* coef;
  void compute_degre ();
public:
  // Construction and conversion operations
  ~POLYNOME () { delete[] coef; }
  POLYNOME ();
  POLYNOME (const CX a);
  POLYNOME (const POLYNOME& a);
  VECTEUR vecteur (ulong dim) const;
  // Standard arithmetic operators
  void operator = (const POLYNOME& a);
  POLYNOME& operator *= (const CX& k);
  POLYNOME& operator += (const POLYNOME& a);
  friend POLYNOME operator + (const POLYNOME& a, const POLYNOME& b);
  friend POLYNOME operator - (const POLYNOME& a, const POLYNOME& b);
  friend POLYNOME operator * (const CX& k, const POLYNOME& a);
  // Other functions
  void mul_xan (const CX& a, ulong n); // Multiplication par (X-a)^n
  void div_xan (const CX& a, ulong n); // Division par (X-a)^n
  void derivee (void);
  // Accessing the internals
  ulong degre () const { return deg - 1; }
  const CX operator [] (const ulong n) const
    { if (n < deg) return coef[n]; else return CX(0); }
  CX& operator [] (const ulong n)
    { if (n < deg) return coef[n]; else
      if (n < len) { coef[n] = CX(0); return coef[n]; } else
      { cerr << "POLYNOME write out of range : " << n << " >= " << len << "\n";
      return coef[0]; } }
};

extern ostream& operator << (ostream& s, const POLYNOME& a);

#endif
