#ifndef H_TOPOLOGIE
#define H_TOPOLOGIE

// On dfinit la classe TOPOLOGIE qui contient la topologie
// en particulier les listes de valences (classe VALENCE)

#include "cx.h" // dfinition de ulong, cerr, ...

class VALENCES
{
  ulong nombre;
  ulong *liste;
public:
  VALENCES (const ulong degre = 0);
  VALENCES (const char* ligne);
  VALENCES (const VALENCES& a);
  ~VALENCES () { delete[] liste; }
  void fusion (ulong i, ulong j);
  void split (ulong i, ulong v);
  void swap (ulong i, ulong j);
  ulong n () const { return nombre; }
  const ulong operator [] (const ulong n) const
    { if (n < nombre) return liste[n]; else
    { cerr << "VALENCES out of range\n"; return nombre; } }
private: // On supprime les oprateurs par dfaut
  VALENCES& operator = (const VALENCES&);
  // void operator = (const VALENCES& a);
};

class TOPOLOGIE
{
  ulong topo_genre, topo_degre;
  VALENCES *topo_faces, *topo_sommets, *topo_aretes, *topo_quatriemes;
  RE *val_quatrieme; // quatrime valeur de ramification entre 0 et 1
  ulong *refcount;
  void copy_if_multiref ();
public:
  enum type { lambda, face, sommet, arete, quatrieme };
  ~TOPOLOGIE ();
  TOPOLOGIE (ulong g, ulong d, VALENCES& f, VALENCES& s, VALENCES& a,
             const RE C = RE(), const VALENCES q = VALENCES());
  // Modificateurs
  void moveC (const RE& newC);
  void fusion (ulong i, ulong j);
  void split (ulong i, ulong v);
  void inversion (ulong mark);
  // Toutes les mthodes ci-dessous sont "const"
  TOPOLOGIE (const TOPOLOGIE& t);
  ulong genre      () const { return topo_genre; }
  ulong degre      () const { return topo_degre; }
  const VALENCES& faces      () const { return *topo_faces; }
  const VALENCES& sommets    () const { return *topo_sommets; }
  const VALENCES& aretes     () const { return *topo_aretes; }
  const VALENCES& quatriemes () const { return *topo_quatriemes; }
  ulong nombrevaleurs () const;
  const RE& C () const { return *val_quatrieme; }
  // Les sommets sont numrots de 0  d+(2-2g) ; accs  leur type :
  ulong valence (ulong i) const;
  type typeramif (ulong i) const;
  const char* variable (ulong i) const;

  friend ostream& operator << (ostream& s, const TOPOLOGIE& t);
  friend class BELYI;
private: // On supprime les oprateurs par dfaut
  TOPOLOGIE& operator = (const TOPOLOGIE&);
};

#endif
