#ifndef H_VECTEUR
#define H_VECTEUR

#include "cx.h"
#include "sys.h"

class VECTEUR
{
  ulong   dim;
  CX    * val;
  ulong * refcount;
  void copy_if_multiref ();
public:
  ~VECTEUR ();
  VECTEUR (ulong s);
  VECTEUR (const VECTEUR& a);

  ulong length () const { return dim; }
  CX& operator [] (const ulong n)
    { copy_if_multiref ();
      if (n < dim) return val[n]; else
    { cerr << "VECTEUR write out of range : " << n << " >= " << dim << "\n";
      return val[0]; } }
  const CX& operator [] (const ulong n) const
    { if (n < dim) return val[n]; else
    { cerr << "VECTEUR read out of range : " << n << " >= " << dim << "\n";
      return val[0]; } }

  void operator = (const VECTEUR& a);
  VECTEUR& operator += (const CX& c); // concatnation

  VECTEUR& operator *= (const CX& c);
  VECTEUR& operator /= (const CX& c);
  VECTEUR& operator += (const VECTEUR& a);
  VECTEUR& operator -= (const VECTEUR& a);
//  friend VECTEUR operator + (const VECTEUR& a, const VECTEUR& b);
//  friend VECTEUR operator - (const VECTEUR& a, const VECTEUR& b);
//  friend VECTEUR operator - (const VECTEUR& a);
};

extern ostream& operator << (ostream& s, const VECTEUR& a);
extern RE norm (const VECTEUR& a);

class MATRICE
{
  ulong   nbl;
  ulong   nbc;
  CX    * val;
  ulong * refcount;
  void copy_if_multiref ();
public:
  ~MATRICE ();
  MATRICE (ulong l, ulong c);
  MATRICE (const MATRICE& a);

  ulong lig () const { return nbl; }
  ulong col () const { return nbc; }
  CX& operator () (const ulong l, const ulong c)
    { copy_if_multiref ();
      if ((c < nbc)&&(l < nbl)) return val[c+l*nbc]; else
    { cerr << "MATRICE write out of range : ("
           << l << "x" << c << ") >= (" << nbl << "x" << nbc <<")\n";
      return val[0]; } }
  const CX& operator () (const ulong l, const ulong c) const
    { if ((c < nbc)&&(l < nbl)) return val[c+l*nbc]; else
    { cerr << "MATRICE read out of range : ("
           << l << "x" << c << ") >= (" << nbl << "x" << nbc <<")\n";
      return val[0]; } }

  void operator = (const MATRICE& a);
  VECTEUR operator * (const VECTEUR& x) const;
  VECTEUR mul (const VECTEUR& x) const;  // Multiplication par un vecteur
};

extern ostream& operator << (ostream& s, const MATRICE& a);

class MCARREE : public MATRICE
{
public:
  MCARREE (ulong d) : MATRICE (d, d) { }
  ulong dim () const { return lig(); }
};

extern CX matinv (const MCARREE& matrice, MCARREE& inverse);

#endif
